#' vectorize.list
#' 
#' Change a `list` in a `vectdata` object.
#' Inner function called by [read.swe()]. Vectdata contains at least dimension
#' x.coords, y.coords, and data (this later name can be changed through a parameter).
#' Data are in a 2D matrix of dimension space x time. Relations between the position 
#' along the first dimension of the data matrix and the real coordinates stored
#' in x.coords and y.coords are tracked using the x.indices and x.indices vectors.
#'
#' @param data The list containing the data
#' @param matrix.vars The name of the variables used to store the proper data.
#' This name is also used in the output `vectdata` object
#' 
#' @return A `vectdata` object containing the infromation of the input list
#' @export
#' @author Adrien Michel, 2023 (MeteoSwiss & WSL/SLF)
vectorize.list <- function(data, matrix.vars="data")
{
  # Vectorize matrix elements
  if("time" %in% names(data)) {
    for(var in c(matrix.vars))
    {
      dim(data[[var]]) <- c(length(data$x.coords)*length(data$y.coords),length(data$time))
    }
  } else {
    for(var in c(matrix.vars))
    {
      dim(data[[var]]) <- c(length(data$x.coords)*length(data$y.coords),1)
    }
  }
  # Vectorize other elements
  for(var in names(data)) {
    if(!(var %in% c(matrix.vars))){
      data[[var]] <- c(data[[var]])
    }
  }
  if(!"x.coords" %in% names(data) & !"y.coords" %in% names(data)) {
    stop("Both x.coords and y.coords variables must exist in the list to be vectorized")
  }
  # Create x and y indices vectors
  data$x.indices=rep(seq_along(data$x.coords), times=length(data$y.coords))
  data$y.indices=rep(seq_along(data$y.coords), each=length(data$x.coords))
  class(data)=c("vectdata","list")
  return(data)
}



