# Internal function computing the temporal clustering
compute.temporal.cluster.internal<- function(timestamps, time, temporal.step)
{
  lower.temporal.bound <- (time-temporal.step) %% 365
  higher.temporal.bound <- (time+temporal.step) %% 365
  if(lower.temporal.bound < higher.temporal.bound)
  {
    time.cluster <- seq(lower.temporal.bound, higher.temporal.bound)
  }else if(lower.temporal.bound == higher.temporal.bound){
    time.cluster <- time
  }else
  {
    time.cluster <- c(seq(lower.temporal.bound,366), seq(1,higher.temporal.bound))
  }
  time.cluster <- which(as.integer(strftime(timestamps, format = "%j")) %in% time.cluster)
  return(time.cluster)
}


#' compute.temporal.cluster
#'
#' Computes the temporal groups used for the quantile mapping
#'
#' @param data.training The dataset containing the timesteps that will be used to
#' train the qunatile mapping (a `vectData` object, see [vectorize.list()])
#' @param params.qm Parameters for the quantile mapping, see [train.qm()]
#' @return A list containing one entry per day of the year, each entry containing a 
#' vector indicating the number of the timesteps belonging the the temporal group 
#' for this day of the year
#'
#' @author Adrien Michel, 2023 (MeteoSwiss & WSL/SLF)
#' @export
compute.temporal.cluster <- function(data, params.qm, output.file=NULL)
{
  if(!is.null(output.file)) {
    if(!dir.exists(dirname(output.file))) {
      stop(paste0("Output directory '",dirname(output.file),"' does not exist"))
    }
    else if(file.access(".", mode=2)==-1) {
      stop(paste0("Output directory '",dirname(output.file),"' seems to benot writable"))
    }
  }
  if(is.null(params.qm$spatial.step)) {
    stop("spatial.step must to exist in params.qm")
  }
  #Remove useless data
  data$data = NULL

  timestamps <- data$time.date

  `%do%` <- foreach::`%do%`
  temporal.cluster <- foreach::foreach(time=c(1:366)) %do% {
    compute.temporal.cluster.internal(timestamps, time,
                                    params.qm$temporal.step)
  }

  if(!is.null(output.file)) {
    saveRDS(temporal.cluster,paste0(output.file))
  }
  return(temporal.cluster)
}
